﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows.Forms;

namespace LINQtoCSVTest
{
	public partial class Form1 : Form
	{
		private List<CSVDATA> FDataList = null;
		private string FCsvFileName;

		public Form1()
		{
			InitializeComponent();
		}

		private void mnuFileOpen_Click(object sender, EventArgs e)
		{
			var dlg = new OpenFileDialog();
			dlg.InitialDirectory = System.IO.Directory.GetCurrentDirectory();
			dlg.Filter = "CSV ファイル|*.csv|すべてのファイル|*.*";

			if (dlg.ShowDialog() == DialogResult.OK)
			{
				FCsvFileName = dlg.FileName;
				this.ReadCsvFile(FCsvFileName);
				statusLabel1.Text = FCsvFileName;
			}
		}

		private void mnuFileOutput_Click(object sender, EventArgs e)
		{
			if (FDataList == null)
				return;

			this.Cursor = Cursors.WaitCursor;
			var context = new LINQtoCSV.CsvContext();
			var description = new LINQtoCSV.CsvFileDescription();
			description.DetectEncodingFromByteOrderMarks = false;
			description.EnforceCsvColumnAttribute = true;
			description.FileCultureInfo = System.Globalization.CultureInfo.CurrentCulture; // デフォルト
			description.FileCultureName = "ja-jp";
			description.FirstLineHasColumnNames = true; // デフォルト
			description.MaximumNbrExceptions = 100; // デフォルト
			description.QuoteAllFields = false; // デフォルト
			description.SeparatorChar = ','; // デフォルト
			description.TextEncoding = System.Text.Encoding.UTF8; // デフォルト

			try
			{
				// 出力ファイル名は入力ファイル名の先頭に "_" を付け、カレントディレクトリに出力する
				context.Write<CSVDATA>(FDataList, "_" + System.IO.Path.GetFileName(FCsvFileName), description);
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message);
			}

			this.Cursor = Cursors.Default;
		}

		private void mnuFileExit_Click(object sender, EventArgs e)
		{
			this.Close();
		}

		private void ReadCsvFile(string fileName)
		{
			this.Cursor = Cursors.WaitCursor;

			var context = new LINQtoCSV.CsvContext();
			var description = new LINQtoCSV.CsvFileDescription();
			description.DetectEncodingFromByteOrderMarks = false;
			description.EnforceCsvColumnAttribute = true;
			description.FileCultureInfo = System.Globalization.CultureInfo.CurrentCulture; // デフォルト
			description.FileCultureName = "ja-jp";
			description.FirstLineHasColumnNames = true; // デフォルト
			description.MaximumNbrExceptions = 100; // デフォルト
			description.QuoteAllFields = false; // デフォルト
			description.SeparatorChar = ','; // デフォルト
			description.TextEncoding = System.Text.Encoding.UTF8; // デフォルト

			IEnumerable<CSVDATA> rows = null;

			try
			{
				rows = context.Read<CSVDATA>(fileName, description);
				FDataList = rows.ToList<CSVDATA>();
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message);
				this.Cursor = Cursors.Default;
				return;
			}

			listView1.View = View.Details;
			listView1.FullRowSelect = true;
			listView1.LabelEdit = true;
			listView1.Columns.Add("氏名", 100, HorizontalAlignment.Left);
			listView1.Columns.Add("生年月日", 100, HorizontalAlignment.Left);
			listView1.Columns.Add("住所", 100, HorizontalAlignment.Left);
			listView1.Columns.Add("没年時の年齢", 100, HorizontalAlignment.Right);

			listView1.BeginUpdate();

			foreach (CSVDATA data in FDataList)
			{
				var item = listView1.Items.Add(data.氏名);
				item.SubItems.Add(String.Format("{0:D}", data.生年月日));
				item.SubItems.Add(data.住所);
				item.SubItems.Add(String.Format("{0}", data.没年時の年齢));
			}

			listView1.EndUpdate();

			this.Cursor = Cursors.Default;
		}

		private void mnuEditSort_Click(object sender, EventArgs e)
		{
			if (FDataList == null)
			{
				MessageBox.Show("CSV ファイルを読み込んでください。");
				return;
			}

			// 生年月日でソート
			var sortedData = from data in FDataList
				               orderby data.生年月日
								       select data;

			listView1.Items.Clear();
			listView1.BeginUpdate();

			foreach (CSVDATA data in sortedData)
			{
				var item = listView1.Items.Add(data.氏名);
				item.SubItems.Add(String.Format("{0:D}", data.生年月日));
				item.SubItems.Add(data.住所);
				item.SubItems.Add(String.Format("{0}", data.没年時の年齢));
			}

			FDataList = sortedData.ToList<CSVDATA>();
			listView1.EndUpdate();
		}

		private void listView1_DoubleClick(object sender, EventArgs e)
		{
			int index = listView1.SelectedIndices[0]; // DoubleClick した位置のインデックス

			var form = new EditData();
			form.RowData = FDataList[index];

			if (form.ShowDialog() == DialogResult.OK)
			{
				FDataList[index] = form.RowData;

				listView1.Items.Clear();
				listView1.BeginUpdate();

				foreach (CSVDATA data in FDataList)
				{
					var item = listView1.Items.Add(data.氏名);
					item.SubItems.Add(String.Format("{0:D}", data.生年月日));
					item.SubItems.Add(data.住所);
					item.SubItems.Add(String.Format("{0}", data.没年時の年齢));
				}

				listView1.EndUpdate();
			}
		}
	} // end of Form1 class
} // end of namespace
